from .balance import GetBalance
from .base import APIMethod
from .order import GetOrders, OrderProxy, GetOrderPrice
from .other import GetCountries, GetIPs, GetIPsCount
from .proxy import GetProxyList, RenewProxy

__all__ = ['APIMethod', 'GetBalance', 'GetProxyList', 'GetOrders',
           'RenewProxy', 'OrderProxy', 'GetOrderPrice', 'GetCountries',
           'GetIPs', 'GetIPsCount']



from .balance import Balance
from .countries import Countries
from .order import OrderPrice, Order
from .enum import ProxyStatus, ProxyProtocol, ProxyType, ProxyFormat
from .ips import IPs, IPsCount
from .proxy_list import ProxyList, ProxyInfo

__all__ = ['Balance', 'ProxyStatus', 'ProxyProtocol', 'ProxyType',
           'Countries', 'ProxyFormat', 'IPs', 'IPsCount', 'OrderPrice', 'ProxyList',
           'Order', 'ProxyInfo']






from typing import List

from pydantic import BaseModel, validator


class Cities(BaseModel):
    id: int
    name: str


class Countries(BaseModel):
    code: str
    name: str
    cities: List[Cities]



from .base import APIMethod


class GetOrders(APIMethod):
    http_method = 'GET'
    path = '/orders/'


class OrderProxy(APIMethod):
    http_method = 'POST'
    path = '/new-order/'


class GetOrderPrice(APIMethod):
    http_method = 'POST'
    path = '/new-order-amount/'



from datetime import datetime
from typing import List, Union, Optional

from pydantic import BaseModel, Field

from .enum import ProxyType, ProxyProtocol


class PriceData(BaseModel):
    ip_list: List[str]
    period: int
    country: str
    type: ProxyType
    ip_version: ProxyProtocol
    quantity: int


class OrderPrice(BaseModel):
    price: float = Field(..., alias='amount')
    data: PriceData


class Order(BaseModel):
    id: int
    date: datetime



import abc
from typing import Optional, Type

from aiohttp import ClientSession, ContentTypeError

from ..methods.base import APIMethod
from ...exceptions import APIError


class BaseAPIClient(abc.ABC):
    BASE_URL = 'https://panel.proxyline.net/api'

    def __init__(self, api_key: str):
        self.api_key = api_key

    async def _send_request(self, method: Type[APIMethod], params: Optional[dict] = None) -> dict:
        """
        Send request to the API
        :param method: API Method
        :param params: Parameters
        :return: JSON
        """
        async with ClientSession() as session:
            params = self._delete_none(params or {})
            request_url = self._get_request_url(method)
            if method.http_method == 'GET':
                response = await session.get(request_url, params=params)
            else:
                response = await session.post(request_url, data=params)
            try:
                json = await response.json()
            except ContentTypeError:
                raise APIError('Blocked by website ddos guard.')
            if response.status in [403, 400]:
                if json.get('detail'):
                    APIError.detect(json['detail'])
                else:
                    APIError.detect(str(json))
            return json

    def _get_request_url(self, method: Type[APIMethod]) -> str:
        """
        Get url to send a request
        :param method: Method
        :return: URL
        """
        return f'{self.BASE_URL}{method.path}?api_key={self.api_key}'

    def _delete_none(self, _dict: dict) -> dict:
        """Delete None values recursively from all of the dictionaries"""
        for key, value in list(_dict.items()):
            if isinstance(value, dict):
                self._delete_none(value)
            elif value is None:
                del _dict[key]
            elif isinstance(value, list):
                for v_i in value:
                    if isinstance(v_i, dict):
                        self._delete_none(v_i)

        return _dict

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        return None



from enum import Enum
from typing import Union


class ProxyStatus(str, Enum):
    ACTIVE = "active"
    EXPIRED = "expired"
    DELETED = "deleted"


class ProxyType(Enum):
    DEDICATED = 'dedicated'
    SHARED = 'shared'


class ProxyProtocol(int, Enum):
    IPv4 = 4
    IPv6 = 6


class ProxyFormat(str, Enum):
    HTTP = 'txt-http'
    SOCKS5 = 'txt-socks5'
    CUSTOM_HTTP = 'custom-http'
    CUSTOM_SOCKS5 = 'custom-socks5'



from aioproxyline.core.methods import APIMethod


class GetCountries(APIMethod):
    http_method = 'GET'
    path = '/countries/'


class GetIPs(APIMethod):
    http_method = 'GET'
    path = '/ips/'


class GetIPsCount(APIMethod):
    http_method = 'GET'
    path = '/ips-count/'



from abc import ABC
from datetime import datetime
from typing import Optional, List, Union

from pydantic import parse_obj_as

from .methods import GetBalance, GetProxyList, GetOrders, RenewProxy, OrderProxy, GetCountries, GetIPs, GetIPsCount, \
    GetOrderPrice
from ..core.abc import BaseAPIClient
from ..types import Balance, ProxyStatus, ProxyType, ProxyProtocol, Countries, ProxyFormat, IPs, IPsCount, OrderPrice, \
    ProxyList, ProxyInfo, Order


class ProxyLine(BaseAPIClient, ABC):
    def __init__(self, api_key: str):
        super().__init__(api_key)

    async def get_balance(self) -> Balance:
        """
        Get balance
        :return: Account balance information
        :rtype: Balance
        """
        result = await self._send_request(GetBalance)
        return Balance(**result)

    async def get_proxy_list(self, status: Optional[ProxyStatus] = None, proxy_type: Optional[ProxyType] = None,
                             ip_version: Optional[ProxyProtocol] = None, country: Optional[str] = None,
                             date_end_after: Optional[datetime] = None, date_end_before: Optional[datetime] = None,
                             date_after: Optional[datetime] = None, date_before: Optional[datetime] = None,
                             orders: Optional[Union[List[int], int]] = None,
                             proxy_format: Optional[ProxyFormat] = None,
                             limit: Optional[int] = 200,
                             offset: Optional[int] = 0) -> ProxyList:
        """
        Get proxy list
        :param status: Status
        :param proxy_type: Type
        :param ip_version: IP version
        :param country: Country
        :param date_after: Date after
        :param date_before: Date before
        :param date_end_after: Date end after
        :param date_end_before: Date end before
        :param orders: Orders
        :param proxy_format: Format
        :param limit: Limit
        :param offset: Offset
        :return: List of proxies
        :rtype: ProxyList
        """
        data = {
            'status': status.value if status else None,
            'type': proxy_type.value if proxy_type else None,
            'ip_version': ip_version.value if ip_version else None,
            'country': country,
            'date_after': date_after,
            'date_before': date_before,
            'date_end_after': date_end_after,
            'date_end_before': date_end_before,
            'orders': orders,
            'format': proxy_format.value if proxy_format else None,
            'limit': limit,
            'offset': offset
        }
        result = await self._send_request(GetProxyList, data)
        modified_result = self._modify_proxy_list_json(result)
        return ProxyList(**modified_result)

    async def get_orders(self, date_after: Optional[datetime] = None,
                         date_before: Optional[datetime] = None) -> List[Order]:
        """
        Get Orders
        :param date_after: Date after
        :param date_before: Date before
        :return: List of orders
        :rtype: List[Order]
        """
        data = {
            'date_after': date_after,
            'date_before': date_before
        }
        result = await self._send_request(GetOrders, data)
        return parse_obj_as(List[Order], result)

    async def renew_proxy(self, proxy: Union[List[int], int], period: int,
                          coupon: Optional[str] = None) -> List[ProxyInfo]:
        """
        Renew proxy
        :param proxy: Proxy ID
        :param period: Renew period
        :param coupon: Coupon
        :return: Proxy Information
        :rtype: List[ProxyInfo]
        """
        data = {
            'proxies': proxy,
            'period': period,
            'coupon': coupon
        }
        result = await self._send_request(RenewProxy, data)
        modified_json = self._modify_renew_proxy_json(result)
        return parse_obj_as(List[ProxyInfo], modified_json)

    async def order_proxy(self, proxy_type: ProxyType, ip_version: ProxyProtocol, country: str,
                          period: int, quantity: int, coupon: Optional[str] = None,
                          ip_list: Optional[List[int]] = None) -> List[ProxyInfo]:
        """
        Order proxy
        :param quantity: Quantity
        :param ip_list: IP List received from ips method
        :param proxy_type: Proxy Type
        :param ip_version: IP version
        :param country: Country
        :param period: Period
        :param coupon: Coupon
        :return: Proxy Information
        :rtype: List[ProxyInfo]
        """
        data = {
            'type': proxy_type.value,
            'ip_version': ip_version.value,
            'country': country,
            'period': period,
            'ip_list': ip_list,
            'quantity': quantity,
            'coupon': coupon
        }
        result = await self._send_request(OrderProxy, data)
        return parse_obj_as(List[ProxyInfo], result)

    async def get_order_price(self, proxy_type: ProxyType, ip_version: ProxyProtocol, country: str,
                              period: int, quantity: int, coupon: Optional[str] = None,
                              ip_list: Optional[List[int]] = None) -> OrderPrice:
        """
        Get Proxy Order Price
        :param quantity: Quantity
        :param ip_list: IP List received from ips method
        :param proxy_type: Proxy Type
        :param ip_version: IP version
        :param country: Country
        :param period: Period
        :param coupon: Coupon
        :return: Order Price
        :rtype: OrderPrice
        """
        data = {
            'type': proxy_type.value,
            'ip_version': ip_version.value,
            'country': country,
            'period': period,
            'ip_list': ip_list,
            'quantity': quantity,
            'coupon': coupon
        }
        result = await self._send_request(GetOrderPrice, data)
        return OrderPrice(**result)

    async def get_countries(self) -> List[Countries]:
        """
        Get countries
        :return: Countries info
        :rtype: List[Countries]
        """
        result = await self._send_request(GetCountries)
        return parse_obj_as(List[Countries], result)

    async def get_ips(self, proxy_type: ProxyType, ip_version: ProxyProtocol, country: str,
                      city: Optional[str] = None) -> List[IPs]:
        """
        Get IPs
        :param city: City
        :param proxy_type: Proxy Type
        :param ip_version: IP version
        :param country: Country
        :return: List of IPs
        :rtype: List[IPs]
        """
        data = {
            'type': proxy_type.value,
            'ip_version': ip_version.value,
            'country': country,
            'city': city
        }
        result = await self._send_request(GetIPs, data)
        return parse_obj_as(List[IPs], result)

    async def get_ips_count(self, proxy_type: ProxyType, ip_version: ProxyProtocol, country: str,
                            city: Optional[str] = None) -> IPsCount:
        """
        Get IPs Count
        :param city: City
        :param proxy_type: Proxy Type
        :param ip_version: IP version
        :param country: Country name
        :return: IPs Count
        :rtype: IPsCount
        """
        data = {
            'type': proxy_type.value,
            'ip_version': ip_version.value,
            'country': country,
            'city': city
        }
        result = await self._send_request(GetIPsCount, data)
        return IPsCount(**result)

    @staticmethod
    def _modify_proxy_list_json(json: dict) -> dict:
        for result in json['results']:
            result['type'] = 'dedicated' if result['type'] == "1" else 'shared'
        return json

    @staticmethod
    def _modify_renew_proxy_json(json: dict) -> dict:
        for result in json:
            result['type'] = 'dedicated' if result['type'] == "1" else 'shared'
        return json



from datetime import datetime
from typing import List, Union, Optional

from pydantic import BaseModel, Field

from .enum import ProxyProtocol, ProxyType


class ProxyTags(BaseModel):
    id: int
    name: str


class ProxyAccessIPs(BaseModel):
    id: int
    ip: str = Field(..., alias='value')
    name: Optional[str]


class ProxyInfo(BaseModel):
    id: int
    ip: str
    internal_ip: Optional[str]
    http_port: int = Field(..., alias='port_http')
    socks5_port: int = Field(..., alias='port_socks5')
    user: str
    username: str
    password: str
    order_id: int
    type: ProxyType
    ip_version: ProxyProtocol
    country: str
    date: datetime
    date_end: datetime
    tags: Optional[List[ProxyTags]]
    access_ips: Optional[List[ProxyAccessIPs]]


class ProxyList(BaseModel):
    count: int
    next: Optional[str]
    previous: Optional[str]
    results: List[ProxyInfo]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aioproxyline',
 'aioproxyline.core',
 'aioproxyline.core.abc',
 'aioproxyline.core.methods',
 'aioproxyline.exceptions',
 'aioproxyline.types']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0', 'pydantic>=1.10.2,<2.0.0']

setup_kwargs = {
    'name': 'aioproxyline',
    'version': '1.1.1',
    'description': 'Asynchronous wrapper to interact with proxyline.net API',
    'long_description': '[![Downloads](https://pepy.tech/badge/aioproxyline)](https://pepy.tech/project/aioproxyline)\n[![Downloads](https://pepy.tech/badge/aioproxyline/month)](https://pepy.tech/project/aioproxyline)\n[![Downloads](https://pepy.tech/badge/aioproxyline/week)](https://pepy.tech/project/aioproxyline)\n[![Code Quality Score](https://api.codiga.io/project/34554/score/svg)](https://api.codiga.io/project/34554/score/svg)\n[![Code Grade](https://api.codiga.io/project/34554/status/svg)](https://api.codiga.io/project/34554/status/svg)\n\n## 🔗 Links\n* 🎓 **Documentation:** [*CLICK*](https://aioproxyline.readthedocs.io/en/latest/)\n* 🖱️ **Developer contacts:** [![Dev-Telegram](https://img.shields.io/badge/Telegram-blue.svg?style=flat-square&logo=telegram)](https://t.me/marple_tech)\n## 🐦 Dependencies  \n\n| Library  |                       Description                       |\n|:--------:|:-------------------------------------------------------:|\n| aiohttp  | Asynchronous HTTP Client/Server for asyncio and Python. |\n| pydantic |                   JSON Data Validator                   |\n\n---\n',
    'author': 'Marple',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/marple-git/aioproxyline',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
